
#include <iostream>
#include <cmath>

#include "CDataHime.hh"

ClassImp(CDataHime)

CDataHime::CDataHime(){;}

CDataHime::CDataHime(const char* name, const char* title) : CData(name, title)
{;}

CDataHime::CDataHime(const TString& name, const TString& title) : CData(name, title)
{;}

CDataHime::CDataHime(const CDataHime &rhs) : CData(), fDequeOfOneData(rhs.fDequeOfOneData)
{;}

CDataHime& CDataHime::operator=(const CDataHime &rhs)
{
  fDequeOfOneData = rhs.fDequeOfOneData;

  fX = rhs.fX;
  fP = rhs.fP;
  fPrp = rhs.fPrp;

  return *this;
}

CDataHime::~CDataHime(){;}

CDataHime::OneData* CDataHime::Find(Int_t idlayer, Int_t idmodule)
{
  std::deque<CDataHime::OneData>::iterator it = fDequeOfOneData.begin();  
  while(it != fDequeOfOneData.end()){
    if(it->fId[0] == idlayer && it->fId[1] == idmodule) return &(*it);
    ++it;
  }

  return 0;
}

void CDataHime::Reset()
{
  CData::Reset();
  fDequeOfOneData.clear();
}

void CDataHime::ClearMomentum()
{
  fP.SetXYZT(0,0,0,0);
  fP2.SetXYZT(0,0,0,0);
}

void CDataHime::CalMomentum() throw()
{
  double m0 = 939.56533;
  double c = 29.9792458;

  TVector3 beta = 1./(fX.T()*c)*fX.Vect();
  TVector3 momentum = m0*1./sqrt(1.-beta.Mag()*beta.Mag())*beta;

  if(!std::isfinite(momentum.Px()) ||
     !std::isfinite(momentum.Py()) ||
     !std::isfinite(momentum.Pz())){
    std::cout << "\n" << "calculated momentum is not finite number" << "\n" << std::endl;
    throw;
  }

  fP.SetVectM(momentum, m0);
}

void CDataHime::CalMomentum2() throw()
{
  double m0 = 939.56533;
  double c = 29.9792458;

  TVector3 beta = 1./(fX2.T()*c)*fX2.Vect();
  TVector3 momentum = m0*1./sqrt(1.-beta.Mag()*beta.Mag())*beta;

  if(!std::isfinite(momentum.Px()) ||
     !std::isfinite(momentum.Py()) ||
     !std::isfinite(momentum.Pz())){
    std::cout << "\n" << "calculated momentum is not finite number" << "\n" << std::endl;
    throw;
  }

  fP2.SetVectM(momentum, m0);
}

void CDataHime::OneData::Print()
{
  std::cout << "  Layer ID:" << fId[0] << std::endl
	    << "  Module ID:" << fId[1] << std::endl
	    << "  TDC Left:" << fTL << std::endl
	    << "  TDC right:" << fTR << std::endl
	    << "  ADC Left:" << fAL << std::endl
	    << "  ADC Right:" << fAR << std::endl
	    << "  Time:" << fT << std::endl
	    << "  Energy:" << fE << std::endl
	    << "  Position:"
	    << fX.X() << ", "
	    << fX.Y() << ", "
	    << fX.Z() << std::endl;
}
