
#include "CMagnet.hh"
#include "CMapFunction2D.hh"
#include "CParticle.hh"
#include "CParticleDefinition.hh"

#include "TMath.h"

#include <string>
#include <vector>
#include <cmath>

ClassImp(CMagnet)

CMagnet::CMagnet()
{/* nothing */;}

CMagnet::~CMagnet()
{/* nothing */;}

int CMagnet::CalTrack(CMapFunction2D* map2d,
		      std::vector<double>& vx, std::vector<double>& vy,
		      double theta,
		      double rstep, double thetastep, double rangelimit,
		      double mass, double beta, double q)
{
  if(vx.empty() || vy.empty()){
    std::cout << "vector need initial value" << std::endl;
    return -1;
  }

  double coef = mass*1e6/TMath::C()/q*beta/sqrt(1-beta*beta);

  double posx = vx.back();
  double posy = vy.back();

  double ddirxdef = sin(thetastep);
  double ddirydef = 1-cos(thetastep);

  double xmin = map2d->GetXmin();
  double xmax = map2d->GetXmax();
  double ymin = map2d->GetYmin();
  double ymax = map2d->GetYmax();

  double thetastep_loop = theta;

  double rangecounterlimit = sqrt(map2d->GetIntervalx()*map2d->GetIntervaly());

  double range(0);
  double rangecounter(0);

  if(q == 0 || beta == 1){
    while(1){
      posx += rstep*cos(theta);
      posy += rstep*sin(theta);
      rangecounter += sqrt(posx*posx+posy*posy);
      if(rangecounter > rangecounterlimit/10){
	vx.push_back(posx);
	vy.push_back(posy);
	rangecounter = 0;
      }
      if(posx < xmin || posx >= xmax || posy < ymin || posy >= ymax
	   || range > rangelimit) break;
    }
  }else{
    while(1){
      double dirx = cos(thetastep_loop);
      double diry = sin(thetastep_loop);

      double b =  map2d->MapFunction(posx, posy);
      double r, ddirx(ddirxdef), ddiry(ddirydef);
      if(b != 0){
	double rtemp = coef/b;
	r = std::abs(rtemp);
	double thetastep_r = rstep/r;
	if(thetastep < thetastep_r){
	  thetastep_loop += (std::signbit(rtemp)) ? thetastep : -thetastep;
	}else{
	  thetastep_loop += (std::signbit(rtemp)) ? thetastep_r : -thetastep_r;
	  ddirx = sin(thetastep_r);
	  ddiry = 1-cos(thetastep_r);
	}
      }else{
	r = rstep;
	ddirx = 1;
	ddiry = 0;
      }
      double xstep = r*(dirx*ddirx-diry*ddiry);
      double ystep = r*(diry*ddirx+dirx*ddiry);
      posx += xstep;
      posy += ystep;
      double rangetemp = sqrt(xstep*xstep+ystep*ystep);
      range += rangetemp;
      rangecounter += rangetemp;
      if(rangecounter > rangecounterlimit/10){
	vx.push_back(posx);
	vy.push_back(posy);
	rangecounter = 0;
      }
      if(posx < xmin || posx >= xmax || posy < ymin || posy >= ymax
	   || range > rangelimit) break;
    }
  }
  
  return 0;
}
