
#include <cmath>

#include "TF1.h"
#include "TMath.h"

#include "FFunction.hh"

Double_t VirtualResonance(Double_t xx, Double_t a, Double_t r0,
			  Double_t Eb, Double_t ma, Double_t mb)
{
  Double_t hbarc(197.0);

  Double_t mcn(ma*mb/(ma+mb));

  Double_t pcn(sqrt(xx*(xx+2.0*mcn)));
  Double_t k(sqrt(2.0*mcn*Eb));
  Double_t delta(atan(pcn/(-1.0/(a/hbarc)+0.5*(r0/hbarc)*pcn*pcn)));

  Double_t result = sqrt(pcn)*(cos(delta)+k*sin(delta)/pcn)/(k*k+pcn*pcn);

  return result*result;
}

Double_t VirtualResonance(Double_t *x, Double_t *par)
{
  Double_t xx = x[0];
  Double_t a  = par[0];
  Double_t r0 = par[1];
  Double_t Eb = par[2];
  Double_t ma = par[3];  
  Double_t mb = par[4];

  return VirtualResonance(xx, a, r0, Eb, ma, mb);
}

Double_t BrietWigner(Double_t xx, Double_t er, Double_t gamma)
{
  Double_t g1(gamma*pow(er, -3.0/2.0));
  Double_t g2(0);//not exaxt;
  Double_t Gamma_Er(g1*pow(xx, 3.0/2.0)/(1+g2*xx));

  return Gamma_Er/((er-xx)*(er-xx)+0.25*Gamma_Er*Gamma_Er);
}

Double_t BrietWigner(Double_t *x, Double_t *par)
{
  Double_t xx = x[0];
  Double_t er = par[0];
  Double_t gamma = par[1];

  return BrietWigner(xx, er, gamma);
}

Double_t BrietWigner2(Double_t xx, Double_t k, Double_t er1, Double_t gamma1, Double_t er2, Double_t gamma2)
{
  return k*BrietWigner(xx, er1, gamma1)+BrietWigner(xx, er2, gamma2);
}

Double_t BrietWigner2(Double_t *x, Double_t *par)
{
  Double_t xx = x[0];
  Double_t k = par[0];
  Double_t er1 = par[1];
  Double_t gamma1 = par[2];
  Double_t er2 = par[3];
  Double_t gamma2 = par[4];

  return BrietWigner2(xx, k, er1, gamma1, er2, gamma2);
}

Double_t Gaussian(Double_t xx, Double_t mean, Double_t sigma)
{
  Double_t result;

  if(sigma == 0){
    result = 0;
  }else{
    result = 1./sqrt(2*TMath::Pi())/sigma*exp(-(xx-mean)*(xx-mean)/2./sigma/sigma);
  }

  return result;
}

Double_t Gaussian(Double_t *x, Double_t *par)
{
  Double_t xx = x[0];
  Double_t mean = par[0];
  Double_t sigma = par[1];

  return Gaussian(xx, mean, sigma);
}

Double_t Poisson(Double_t x, Double_t par)
{
  return TMath::Poisson(x, par);
}

Double_t Poisson(Double_t *x, Double_t *par)
{
  Double_t xx = x[0];
  Double_t c = par[0];
  Double_t k = par[1];
  Double_t lambda = par[2];

  return c*k*Poisson(k*xx, lambda);
}

Double_t SquareWave(Double_t xx, Double_t hight, Double_t width, Double_t period)
{
  while(xx < 0) xx += period;    
  while(xx >= period) xx -= period;
    
  return (xx < width) ? hight : 0;
}

Double_t SquareWave(Double_t *x, Double_t *par)
{
  Double_t xx = x[0];
  Double_t hight = par[0];
  Double_t width = par[1];
  Double_t period = par[2];

  return SquareWave(xx, hight, width, period);
}

