#ifndef CSIMNEUT_HH
#define CSIMNEUT_HH

#include "TObject.h"
#include "TVector3.h"

#include <string>
#include <vector>

#include "TSimulationDataNeutronDetector.hh"
#include "TSimulationParameterNeutronDetector.hh"

class TDirectory;
class TFile;
class TH1;

class CParticle;

class CSimNeut
{
public:
  CSimNeut();
  virtual ~CSimNeut();
  
  int StartAnalysis(std::string rootfile);
  int ReadParameters();
  int CreateHist();
  int CreateExpData();
  //  int AnalysisExpData();
  //  int DrawHist();

private:
  //calculate simlation data to experiment data
  void SimToExp(std::vector<TSimulationDataNeutronDetector>* simulationDataNeutronDetectorArray);

  //analysis method for experiment data
  //  void Single(CDataHime* dataexp);
  //  void Track(CDataHime* dataexp);

private:
  TDirectory* fTopDirectory;
  TFile* fFileSim;
  TFile* fFileExp;

  Long64_t fNumberOfEntries;

  CParticle* fProton;
  CParticle* fNeutron;
  CParticle* fChargedParticle;

  TH1* fHIDQL;
  TH1* fHIDQR;
  TH1* fHIDTL;
  TH1* fHIDTR;
  std::vector<TH1*> fHTQ;

  // neutron detector parameters
  TSimulationParameterNeutronDetector fSimPara;
//  Int_t fModuleNumber;
//  Int_t fLayerNumber;
//  Int_t fStuckNumber;
//  TVector3 fDetectorPos;
//  TVector3 fModuleSize;
//  Double_t fModuleGap;
//  Double_t fLayerGap;
//  Double_t fStuckGap;
//  Int_t fRotationMode;

  Int_t fTotalModuleNumber;

  Double_t fHardwareThreshold;
  Double_t fSoftwareThreshold;
  Double_t fTimeResolution;
  Double_t fAttenuationLength;
  Double_t fPropagationVelocity;
  Int_t fMultiplicity;

  TString fAnalysisMethod;

  // beam parameters
  Double_t fStandardKineticEenrgy;

  // temporary
  std::vector<Int_t> fID;
  std::vector<Double_t> fQL, fQR;
  std::vector<Double_t> fTL, fTR;

  ClassDef(CSimNeut, 0)
};

#endif
