#ifndef ANALYZENEUTRCNP2N_HH
#define ANALYZENEUTRCNP2N_HH

#include "TROOT.h"

#include <string>
#include <deque>

#include "CDataHime.hh"

class TVector3;
class TCanvas;
class TDirectory;
class TFile;
class TH1D;
class TH2D;
class TH3D;

//class CDataHime;
class CDataHimeSim;
class CParticle;

class CAnalyzeNeutRCNP2n
{
public:
  CAnalyzeNeutRCNP2n(std::string rootfile);
  ~CAnalyzeNeutRCNP2n();
  
  void Main(std::string prmfile, std::string detector, std::string method, Long64_t nentries=-1);

  int ReadParamaters(std::string prmfile, std::string detector);
  int CreateHist();
  int CreateExpData(std::string method, Long64_t nentries);
  int AnalysisExpData(Long64_t nentries);
  int DrawHist(Long64_t nentries);

private:
  //calculate simlation data to experiment data
  void SimToExp(CDataHimeSim* simdata, CDataHime* dataexp, std::string method);

  //analysis method for experiment data
  //  void Track(CDataHime* dataexp);
  bool Track(std::deque<CDataHime::OneData> *data, double &x, double &y, double &z, double &t);
  //  void Simple(CDataHime* dataexp);
  bool Simple(std::deque<CDataHime::OneData> *data, double &x, double &y, double &z, double &t);
  //  void Niren(CDataHime* dataexp);//not optimized

  void DivideTwo(int counter,
		 std::deque<CDataHime::OneData> *data,
		 std::deque<CDataHime::OneData> *data1,
		 std::deque<CDataHime::OneData> *data2);
  double OneTrackFit(std::deque<CDataHime::OneData> *data, double* parFit);
  double TwoTrackFit(int counter, std::deque<CDataHime::OneData> *data, double* parFit1, double* parFit2);
  //  std::deque<CDataHime::OneData>::iterator
  CDataHime::OneData
  FrontData(std::deque<CDataHime::OneData> *data);
  CDataHime::OneData
  BackData(std::deque<CDataHime::OneData> *data);

  bool CrossTalk(std::deque<CDataHime::OneData> *data1,
		 std::deque<CDataHime::OneData> *data2);
  void NeutronAngle(std::deque<CDataHime::OneData> *data, double &theta, double &phi);

  void EliminateSolitaryPoint(std::deque<CDataHime::OneData> *data, double distance_scale = 1);

  void Grouping(std::deque<CDataHime::OneData> *data,
		std::deque<std::deque<CDataHime::OneData> > *data_array);

private:
  TCanvas *canvas;

  TDirectory *topdir;
  TFile *filesim, *fileexp;

  CParticle *proton;
  CParticle *neutron;

  //hist
  TH1D *hpx, *hpy, *hpz;
  TH2D *hp2x, *hp2y, *hp2z;
  TH2D *hp2xgate, *hp2ygate, *hp2zgate;

  //parameters
  Int_t thrmethod;
  Double_t thr, treso, attlength, vl;
  Int_t rotation;
  Double_t length, vlength, width, depth, gap, zdepth, zdepth_back, shiftx, shifty;
  Int_t multihit;
  Double_t aminmin, ein, sp;
  Double_t step;

  ClassDef(CAnalyzeNeutRCNP2n, 0)
};

#endif
