
#include "TROOT.h"
#include "TFile.h"
#include "TDirectory.h"
#include "TTree.h"
#include "TCanvas.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TH3D.h"
#include "TGraph2DErrors.h"
#include "TVector3.h"
#include "TF1.h"
#include "TFitResult.h"
#include "TFitResultPtr.h"

#include <iostream>
#include <string>
#include <cmath>
#include <deque>
#include <fstream>

#include "CDataHime.hh"
#include "CDataHimeSim.hh"
#include "CKinema.hh"
#include "CParticle.hh"
#include "CParticleDefinition.hh"
#include "FFunction.hh"
#include "CMapFunction1D.hh"
#include "CFold.hh"
#include "CAnalysis.hh"
#include "CXmlManager.hh"

#include "CAnalyzeTool.hh"

ClassImp(CAnalyzeTool)

double CAnalyzeTool::c(29.9792458);

int CAnalyzeTool::GetFirstProtonInformation(CDataHimeSim* data, CParticle* proton)
{
  static std::deque<CDataHimeSim::OneData>::iterator itsim;
  static std::deque<CDataHimeSim::OneData>::iterator itmin;
  itsim = (data->fDequeOfOneData).begin();
  itmin = (data->fDequeOfOneData).end();
  Double_t tmin = 1e+10;
  while(itsim != data->fDequeOfOneData.end()){
    if(itsim->fParticleName == "proton"){
      if(tmin > itsim->fTime){
	tmin = itsim->fTime;
	itmin = itsim;
      }
    }
    ++itsim;
  }

  if(itmin == data->fDequeOfOneData.end()){
    return 0;
  }else{
    proton->p(itmin->fMomentum.x(),
	      itmin->fMomentum.y(),
	      itmin->fMomentum.z());

    return 1;
  }
}

double CAnalyzeTool::GetTotalEnergyLoss(CDataHime* data)
{
  static std::deque<CDataHime::OneData>::iterator it;
  it = (data->fDequeOfOneData).begin();
  double energy=0.;

  while(it != data->fDequeOfOneData.end()){
    energy += it->fE;
    ++it;
  }

  return energy;
}

void CAnalyzeTool::FoldChargedParticle(TVector3 momentum, CParticle* particle,
			 TVector3 dpoverp, double dele, double estrag, double multi_theta)
{
  static TRandom3 random;

  particle->p(momentum.Px(),
	      momentum.Py(),
	      momentum.Pz());

  double detodp_coef = (particle->k()+particle->m())/(particle->k()+2*particle->m())
    /particle->k()*particle->p();
  double sigma[] = {momentum.Mag()*dpoverp.x(),
		    momentum.Mag()*dpoverp.y(),
		    momentum.Mag()*dpoverp.z(),
		    dele*detodp_coef,
		    estrag*detodp_coef,
		    multi_theta*momentum.Pz()};

  //energy loss difference
  momentum.SetZ(momentum.Pz()+random.Uniform(-sigma[3]/2., sigma[3]/2.));
  //multiple scattering and energy stragglling
  momentum.SetXYZ(random.Gaus(momentum.Px(), sigma[5]),
		  random.Gaus(momentum.Py(), sigma[5]),
		  random.Gaus(momentum.Pz(), sigma[4]));
  //detector resolution
  particle->p(random.Gaus(momentum.Px(), sigma[0]),
	      random.Gaus(momentum.Py(), sigma[1]),
	      random.Gaus(momentum.Pz(), sigma[2]));
}

